﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Identity.UI.Services;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using Microsoft.AspNetCore.WebUtilities;
using ModrinkyWebApp.Models;
using ModrinkyWebApp.Models.Repositare;
using SendGrid;
using SendGrid.Helpers.Mail;
using System;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Text.Encodings.Web;
using System.Threading.Tasks;

namespace ModrinkyWebApp.Areas.Identity.Pages.Account
{
    [AllowAnonymous]
    public class ForgotPasswordModel : PageModel
    {
        private readonly UserManager<Uzivatel> _userManager;
        private readonly IEmailSender _emailSender;
        private readonly IUzivatelRepository uzivatelRepository;
        private readonly IUnitOfWork unitOfWork;

        public ForgotPasswordModel(UserManager<Uzivatel> userManager, IEmailSender emailSender, IUnitOfWork unitOfWork)
        {
            this.unitOfWork = unitOfWork;
            _userManager = userManager;
            _emailSender = emailSender;
            uzivatelRepository = unitOfWork.UzivatelRepository;
        }

        [BindProperty]
        public InputModel Input { get; set; }

        public class InputModel
        {
            [Required(ErrorMessage = "Vyplnění emailové adresy je vyžadováno")]
            [RegularExpression(@"^([a-zA-Z0-9_\-\.]+)@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.)|(([a-zA-Z0-9\-]+\.)+))([a-zA-Z]{2,4}|[0-9]{1,3})(\]?)$", ErrorMessage = "Zadejte prosím validní adresu")]
            public string Email { get; set; }
        }

        public async Task<IActionResult> OnPostAsync()
        {
            if (ModelState.IsValid)
            {
                var user = await _userManager.FindByEmailAsync(Input.Email);
                
                if (!uzivatelRepository.VsichniUzivatele().Any(x => x.Email == Input.Email))
                {
                    //Z bezpecnostnich duvodu neni zobrazeno, ze uzivatel neexistuje
                    return RedirectToPage("./ForgotPasswordConfirmation");
                }

                var code = await _userManager.GeneratePasswordResetTokenAsync(user);
                code = WebEncoders.Base64UrlEncode(Encoding.UTF8.GetBytes(code));
                var callbackUrl = Url.Page(
                    "/Account/ResetPassword",
                    pageHandler: null,
                    values: new { area = "Identity", code },
                    protocol: Request.Scheme);

                //uzivateli je odeslan email s odkazem pro resetovani hesla
                ZapomenuteHesloEmail(Input.Email, $"Pro resetování hesla klikněte <a href='{HtmlEncoder.Default.Encode(callbackUrl)}'>sem</a>.").Wait();
                
                return RedirectToPage("./ForgotPasswordConfirmation");
            }

            return Page();
        }

        private static async Task ZapomenuteHesloEmail(string email, string message)
        {
            Random random = new Random();
            const string chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
            string password = new string(Enumerable.Repeat(chars, 8).Select(s => s[random.Next(s.Length)]).ToArray());

            var apiKey = "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";
            var client = new SendGridClient(apiKey);
            var from = new EmailAddress("modrinkytabor@seznam.cz");
            var to = new EmailAddress(email);
            var subject = "Zapomenuté heslo";

            var msg = MailHelper.CreateSingleEmail
                (
                from, to, subject, null, message
                );
            var response = await client.SendEmailAsync(msg);
        }
    }
}