﻿using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Identity.UI.Services;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using Microsoft.Extensions.Logging;
using ModrinkyWebApp.Models;
using ModrinkyWebApp.Models.Repositare;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Threading.Tasks;

namespace ModrinkyWebApp.Areas.Identity.Pages.Account
{
    [Authorize]
    public class RegisterModel : PageModel
    {
        private readonly SignInManager<Uzivatel> _signInManager;
        private readonly UserManager<Uzivatel> _userManager;
        private readonly ILogger<RegisterModel> _logger;
        private readonly IEmailSender _emailSender;
        private readonly IUzivatelRepository uzivatelRepository;
        private readonly IRoleRepository roleRepository;
        private readonly IClenArchivRepository clenArchivRepository;
        private readonly IUnitOfWork unitOfWork;
        private static Random random = new Random();

        public static string RandomString(int length)
        {
            const string chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
            return new string(Enumerable.Repeat(chars, length)
              .Select(s => s[random.Next(s.Length)]).ToArray());
        }

        public RegisterModel(
            UserManager<Uzivatel> userManager,
            SignInManager<Uzivatel> signInManager,
            ILogger<RegisterModel> logger,
            IEmailSender emailSender,

            IUnitOfWork unitOfWork)
        {
            _userManager = userManager;
            _signInManager = signInManager;
            _logger = logger;
            _emailSender = emailSender;
            this.uzivatelRepository = unitOfWork.UzivatelRepository;
            this.roleRepository = unitOfWork.RoleRepository;
            this.clenArchivRepository = unitOfWork.ClenArchivRepository;
            this.unitOfWork = unitOfWork;
        }

        [BindProperty]
        public InputModel Input { get; set; }

        public string ReturnUrl { get; set; }

        public IList<AuthenticationScheme> ExternalLogins { get; set; }

        public class InputModel
        {
            [Required(ErrorMessage = "Vyplnění emailové adresy je vyžadováno")]
            [RegularExpression(@"^([a-zA-Z0-9_\-\.]+)@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.)|(([a-zA-Z0-9\-]+\.)+))([a-zA-Z]{2,4}|[0-9]{1,3})(\]?)$", ErrorMessage = "Zadejte prosím validní adresu")]

            [Display(Name = "Email")]
            public string Email { get; set; }

            [Required(ErrorMessage = "Vyplnění jména je vyžadováno")]
            [StringLength(50, ErrorMessage = "Jméno může být dlouhé maximálně {1} znaků.")]
            public string Jmeno { get; set; }

            [Required(ErrorMessage = "Vyplnění příjmení je vyžadováno")]
            [StringLength(50, ErrorMessage = "Příjmení může být dlouhé maximálně {1} znaků.")]
            public string Prijmeni { get; set; }

            [Required(ErrorMessage = "Vyplnění role je vyžadováno")]
            public int RoleId { get; set; }
        }

        public string VratRoli()
        {
            Guid prihlasenyUserId = Guid.Parse(_userManager.GetUserId(User));
            int prihlasenyRoleId = uzivatelRepository.UzivatelPodleId(prihlasenyUserId).RoleId;
            string prihlasenyRole = roleRepository.RolePodleId(prihlasenyRoleId).Nazev;
            return prihlasenyRole;
        }

        public bool MaPravo()
        {
            if (VratRoli() == "vedoucí" || VratRoli() == "admin") return true;
            else return false;
        }

        public async Task OnGetAsync(string returnUrl = null)
        {
            ReturnUrl = returnUrl;
            ExternalLogins = (await _signInManager.GetExternalAuthenticationSchemesAsync()).ToList();
        }

        public async Task<IActionResult> OnPostAsync(string returnUrl = null)
        {
            Guid currentUserId = Guid.Parse(_userManager.GetUserId(User));
            int prihlasenyRoleId = uzivatelRepository.VsichniUzivatele().FirstOrDefault(x => x.Id == currentUserId).RoleId;
            string prihlasenyRole = roleRepository.RolePodleId(prihlasenyRoleId).Nazev;

            returnUrl = returnUrl ?? Url.Content("~/");
            ExternalLogins = (await _signInManager.GetExternalAuthenticationSchemesAsync()).ToList();

            if (ModelState.IsValid)
            {
                var user = new Uzivatel
                {
                    Id = Guid.NewGuid(),
                    UserName = Input.Email,
                    Email = Input.Email,
                    KrestniJmeno = Input.Jmeno,
                    Prijmeni = Input.Prijmeni,
                    RoleId = Input.RoleId,
                    LockoutEnabled = false
                };

                //pokud neni uzivatel v roli rodic, uzivatel je automaticky pridan duplicitne do archivu pro archivni akce
                if (Input.RoleId == 1 || Input.RoleId == 4 || Input.RoleId == 3)
                {
                    ClenArchiv pom = new ClenArchiv()
                    {
                        KrestniJmeno = user.KrestniJmeno,
                        ClenArchivId = user.Id,
                        Prijmeni = user.Prijmeni
                    };
                    clenArchivRepository.PridaniArchivnihoClena(pom);
                }
                //slouzi pro vygenerovani nahodneho hesla
                string password = RandomString(8);
                var result = await _userManager.CreateAsync(user, password);
                if (result.Succeeded)
                {
                    unitOfWork.Commit();
                    TempData["Zprava"] = "Uživatel úspěšně přidán.";

                    return RedirectToAction("Dokonceni", "Registrace", new { @id = user.Id, password });
                }
                foreach (var error in result.Errors)
                {
                    ModelState.AddModelError(string.Empty, "Tento email je již obsazen.");
                }
            }

            // If we got this far, something failed, redisplay form
            return Page();
        }
    }
}