﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using ModrinkyWebApp.Models;
using ModrinkyWebApp.Models.Repositare;
using ModrinkyWebApp.ViewModels;
using SendGrid;
using SendGrid.Helpers.Mail;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace ModrinkyWebApp.Controllers
{
    [Authorize]
    public class AkceController : Controller
    {
        public static List<Clen> ovlivneniClenove = new List<Clen>();
        public static List<Uzivatel> ovlivneniUzivatele = new List<Uzivatel>();
        private readonly IAkceRepository akceRepository;
        private readonly IClenRepository clenRepository;
        private readonly IDruzinaRepository druzinaRepository;
        private readonly IPovolenaDruzinaRepository povolenaDruzinaRepository;
        private readonly IPrihlaseniArchivRepository prihlaseniArchivRepository;
        private readonly IPrihlaseniNaAkciClenRepository prihlaseniNaAkciClenRepository;
        private readonly IPrihlaseniNaAkciUzivatelRepository prihlaseniNaAkciUzivatelRepository;
        private readonly IRoleRepository roleRepository;
        private readonly IUnitOfWork unitOfWork;
        private readonly UserManager<Uzivatel> userManager;
        private readonly IUzivatelRepository uzivatelRepository;

        public AkceController(
            UserManager<Uzivatel> userManager,
            IUnitOfWork unitOfWork
            )
        {
            this.userManager = userManager;
            this.unitOfWork = unitOfWork;
            akceRepository = unitOfWork.AkceRepository;
            roleRepository = unitOfWork.RoleRepository;
            uzivatelRepository = unitOfWork.UzivatelRepository;
            clenRepository = unitOfWork.ClenRepository;
            povolenaDruzinaRepository = unitOfWork.PovolenaDruzinaRepository;
            prihlaseniNaAkciUzivatelRepository = unitOfWork.PrihlaseniNaAkciUzivatelRepository;
            prihlaseniNaAkciClenRepository = unitOfWork.PrihlaseniNaAkciClenRepository;
            druzinaRepository = unitOfWork.DruzinaRepository;
            prihlaseniArchivRepository = unitOfWork.PrihlaseniArchivRepository;
        }

        //funkce ktera provede zarchivovani probehlych akci
        public void Archivace()
        {
            //zjisteni, zda akce jiz probehla
            foreach (var a in akceRepository.VsechnyAkce())
            {
                if (
                    (
                    a.RokKonani < int.Parse(DatumPrevody.VratRok())
                    )
                    ||

                    (
                    int.Parse(DatumPrevody.MesicCiselne(a.MesicKonani)) < DatumPrevody.VratMesic()
                    && a.RokKonani == int.Parse(DatumPrevody.VratRok())
                    )
                    ||
                    (
                    a.DenKonani < int.Parse(DatumPrevody.VratDen())
                    && int.Parse(DatumPrevody.MesicCiselne(a.MesicKonani)) == DatumPrevody.VratMesic()
                    && a.RokKonani == int.Parse(DatumPrevody.VratRok())
                    )
                   )
                {
                    //nastaveni akce jako archivni
                    a.ArchivniAkce = true;

                    //zruseni prihlaseni vsech clenu a zaroven zarazeni prihlaseni jako archivni
                    foreach (var p in prihlaseniNaAkciClenRepository.VsechnaPrihlaseniNaAkci())
                    {
                        if (p.AkceId == a.AkceId)
                        {
                            PrihlaseniArchiv pom2 = new PrihlaseniArchiv()
                            {
                                AkceArchivId = a.AkceId,
                                ClenArchivId = p.ClenId
                            };
                            prihlaseniArchivRepository.PridaniArchivnihoPrihlaseni(pom2);

                            prihlaseniNaAkciClenRepository.OdebraniPrihlaseni(p.PrihlaseniNaAkciClenId);
                        }
                    }
                    //zruseni prihlaseni vsech uzivatelu a zaroven zarazeni prihlaseni jako archivni
                    foreach (var p in prihlaseniNaAkciUzivatelRepository.VsechnaPrihlaseniNaAkci())
                    {
                        if (p.AkceId == a.AkceId)
                        {
                            PrihlaseniArchiv pom2 = new PrihlaseniArchiv()
                            {
                                AkceArchivId = a.AkceId,
                                ClenArchivId = p.UzivatelId
                            };
                            prihlaseniArchivRepository.PridaniArchivnihoPrihlaseni(pom2);

                            prihlaseniNaAkciUzivatelRepository.OdebraniPrihlaseni(p.PrihlaseniNaAkciUzivatelId);
                        }
                    }
                    
                    unitOfWork.Commit();
                }
            }
        }

        [Authorize]
        public IActionResult DetailAkce(Guid id)
        {
            Archivace();
            //pro usporu vyuzito jiz existujiciho view model pro editace akce
            var editaceAkce = new EditaceAkceViewModel()
            {
                Akce = akceRepository.AkcePodleId(id),
                Druziny = druzinaRepository.VsechnyDruziny().ToList(),
                PovoleneDruziny = povolenaDruzinaRepository.VsechnyPovoleneDruziny().ToList(),
                PrihlaseniNaAkciClen = prihlaseniNaAkciClenRepository.VsechnaPrihlaseniNaAkci().ToList(),
                PrihlaseniNaAkciUzivatel = prihlaseniNaAkciUzivatelRepository.VsechnaPrihlaseniNaAkci().ToList()
            };
            //zjisteni prihlaseneho uzivatele
            if (VratRoli() == "člen" || VratRoli() == "admin" || VratRoli() == "vedoucí")
            {
                Guid prihlasenyUserId = Guid.Parse(userManager.GetUserId(User));
                editaceAkce.Uzivatel = uzivatelRepository.UzivatelPodleId(prihlasenyUserId);
            }
            if (editaceAkce.Akce == null || editaceAkce.Akce.ArchivniAkce)
            {
                return View("StrankaNenalezena");
            }
            return View(editaceAkce);
        }

        public IActionResult EditaceAkce(Guid id)
        {
            Archivace();
            if (!MaPravo())
            {
                return View("PristupZakazan");
            }
            else if (!akceRepository.VsechnyAkce().Any(x => x.AkceId == id))
            {
                return View("StrankaNenalezena");
            }
            //vedouci smi editovat pouze jim vytvorenou akci
            else if (VratRoli() == "vedoucí" &&
                akceRepository.AkcePodleId(id).AutorId != Guid.Parse(userManager.GetUserId(User)))
            {
                return View("PristupZakazan");
            }

            var editaceAkce = new EditaceAkceViewModel()
            {
                Akce = akceRepository.AkcePodleId(id)
            };
            //slouzi pro zjisteni ktere "checkboxy" maji byt defaultne zatrzeny ve formularo
            //pokud byla akce urcena pro tuto druzinu, je checkbox defaultne zaskrtnut
            foreach (var p in editaceAkce.Akce.PovoleneDruziny)
            {
                if (p.AkceId == editaceAkce.Akce.AkceId)
                {
                    switch (p.DruzinaId)
                    {
                        case 1:
                            editaceAkce.PuvodniModrasci = true;
                            break;

                        case 2:
                            editaceAkce.PuvodniFialky = true;
                            break;

                        case 3:
                            editaceAkce.PuvodniDelfini = true;
                            break;

                        case 4:
                            editaceAkce.PuvodniChobotnice = true;
                            break;
                    }
                }
            }

            if (MaPravo())
            {
                //pokud ake jeste neexistuje, presmeruj na vytvoreni nove
                if (editaceAkce.Akce == null)
                {
                    return RedirectToAction("PridaniAkce", editaceAkce);
                }
                //pro editaci je pouzito dvou formularu, aby nedochazelo k validaci jiz pri prvotnim zadani
                return View("EditaceAkce2", editaceAkce);
            }
            else
            {
                return View("PristupZakazan");
            }
        }

        [HttpPost]
        public IActionResult EditaceAkce(EditaceAkceViewModel editaceAkce)
        {
            if (ModelState.IsValid)
            {
                if (!KontrolaExistenceData(editaceAkce.Akce.MesicKonani, editaceAkce.Akce.DenKonani, editaceAkce.Akce.RokKonani))
                {
                    TempData["Zprava"] = "Zadané datum neexistuje";
                    return View("EditaceAkce", editaceAkce);
                }
                else if (!KontrolaAktualnihoData(editaceAkce.Akce.MesicKonani, editaceAkce.Akce.DenKonani, editaceAkce.Akce.RokKonani))
                {
                    TempData["Zprava"] = "Nelze zadat již problěhlé datum ani dnešní den";
                    return View("EditaceAkce", editaceAkce);
                }
                //povolene druziny se nejdrive vsechny smazou a pak se pridaji nove
                povolenaDruzinaRepository.VymazPovoleneDruzinyProAkci(editaceAkce.Akce.AkceId);
                editaceAkce.Akce.PovoleneDruziny.Clear();

                PridaniPovoleneDruziny(editaceAkce.Modrasci, 1, editaceAkce.Akce.AkceId);
                PridaniPovoleneDruziny(editaceAkce.Fialky, 2, editaceAkce.Akce.AkceId);
                PridaniPovoleneDruziny(editaceAkce.Delfini, 3, editaceAkce.Akce.AkceId);
                PridaniPovoleneDruziny(editaceAkce.Chobotnice, 4, editaceAkce.Akce.AkceId);

                if ((editaceAkce.Chobotnice == false && editaceAkce.PuvodniChobotnice == true)
                    || (editaceAkce.Delfini == false && editaceAkce.PuvodniDelfini == true)
                    || (editaceAkce.Fialky == false && editaceAkce.PuvodniFialky == true)
                    || (editaceAkce.Modrasci == false && editaceAkce.PuvodniModrasci == true))
                {
                    ovlivneniClenove.Clear();
                    ovlivneniUzivatele.Clear();

                    Ovlivneni(4, editaceAkce.Chobotnice, editaceAkce.PuvodniChobotnice, editaceAkce.Akce.AkceId);
                    Ovlivneni(3, editaceAkce.Delfini, editaceAkce.PuvodniDelfini, editaceAkce.Akce.AkceId);
                    Ovlivneni(2, editaceAkce.Fialky, editaceAkce.PuvodniFialky, editaceAkce.Akce.AkceId);
                    Ovlivneni(1, editaceAkce.Modrasci, editaceAkce.PuvodniModrasci, editaceAkce.Akce.AkceId);

                    //odeslani emailu s oznamenim, ze kvuli editaci byl uzivatel/clen odhlasen)
                    if (ovlivneniClenove.Count > 0 || ovlivneniUzivatele.Count > 0)
                    {
                        foreach (var o in ovlivneniUzivatele)
                        {
                            OvlivneniMail(o, editaceAkce.Akce.Nazev).Wait();
                        }

                        foreach (var o in ovlivneniClenove)
                        {
                            Uzivatel rodic = uzivatelRepository.UzivatelPodleId(o.SpravceId);
                            OvlivneniMail(rodic, editaceAkce.Akce.Nazev, o.KrestniJmeno).Wait();
                        }

                        unitOfWork.Commit();
                        TempData["Zprava"] = "Akce úspěšně upravena.";

                        return RedirectToAction("SeznamAkci");
                    }
                }
                akceRepository.EditaceAkce(editaceAkce.Akce);

                unitOfWork.Commit();
                TempData["Zprava"] = "Akce úspěšně upravena.";
                return RedirectToAction("SeznamAkci");
            }
            else
            {
                return View("EditaceAkce", editaceAkce);
            }
        }

        public IActionResult KdoJedeNaAkci(Guid Id)
        {
            Archivace();
            if (!MaPravo())
            {
                return View("PristupZakazan");
            }
            else if (!akceRepository.VsechnyAkce().Any(x => x.AkceId == Id))
            {
                return View("StrankaNenalezena");
            }
            var kdoJede = new KdoJedeViewModel()
            {
                Akce = akceRepository.AkcePodleId(Id),
                ClenoveCoJedou = clenRepository.VsichniClenove().ToList(),
                UzivateleCoJedou = uzivatelRepository.VsichniUzivatele().ToList(),
                PovoleneDruzinyNaAkci = povolenaDruzinaRepository.VsechnyPovoleneDruziny().ToList(),
                PrihlaseniNaAkciClen = prihlaseniNaAkciClenRepository.VsechnaPrihlaseniNaAkci().ToList(),
                PrihlaseniNaAkciUzivatel = prihlaseniNaAkciUzivatelRepository.VsechnaPrihlaseniNaAkci().ToList()
            };
            return View(kdoJede);
        }

        public bool MaPravo()
        {
            if (VratRoli() == "vedoucí" || VratRoli() == "admin") return true;
            else return false;
        }

        public IActionResult OdebraniAkce(Guid id)
        {
            if (!akceRepository.VsechnyAkce().Any(x => x.AkceId == id))
            {
                return View("StrankaNenalezena");
            }
            //vedouci muze odebrat akci pouze je-li jejim autorem
            else if (VratRoli() == "vedoucí" &&
                akceRepository.AkcePodleId(id).AutorId != Guid.Parse(userManager.GetUserId(User)))
            {
                return View("PristupZakazan");
            }

            if (MaPravo())
            {
                Archivace();
                return View();
            }
            return View("PristupZakazan");
        }

        [HttpPost]
        public IActionResult OdebraniAkce(Guid id, bool onPost = true)
        {
            //odebrani prihlaseni
            foreach (var p in prihlaseniNaAkciUzivatelRepository.VsechnaPrihlaseniNaAkci())
            {
                if (p.AkceId == id)
                {
                    prihlaseniNaAkciUzivatelRepository.OdebraniPrihlaseni(p.PrihlaseniNaAkciUzivatelId);
                }
            }
            foreach (var p in prihlaseniNaAkciClenRepository.VsechnaPrihlaseniNaAkci())
            {
                if (p.AkceId == id)
                {
                    prihlaseniNaAkciClenRepository.OdebraniPrihlaseni(p.PrihlaseniNaAkciClenId);
                }
            }

            akceRepository.OdebratAkci(id);
            unitOfWork.Commit();
            TempData["Zprava"] = "Akce úspěšně odebrána.";

            return RedirectToAction("SeznamAkci");
        }

        public IActionResult OdhlasitSeZAkce(Guid id, Guid clenId)
        {
            Archivace();
            Guid prihlasenyUserId = Guid.Parse(userManager.GetUserId(User));
            if (!akceRepository.VsechnyAkce().Any(x => x.AkceId == id))
            {
                return View("StrankaNenalezena");
            }
            else if (!prihlaseniNaAkciUzivatelRepository.VsechnaPrihlaseniNaAkci().Any(x => x.UzivatelId == prihlasenyUserId
             && x.AkceId == id) && VratRoli() != "rodič")
            {
                return View("PristupZakazan");
            }
            var prihlasitSeNaAkci = new PrihlasitSeNaAkciViewModel()
            {
                Akce = akceRepository.AkcePodleId(id),
            };
            //zjisteni clenu kteri se mohou prihlasit na akci
            if (VratRoli() == "člen" || VratRoli() == "admin" || VratRoli() == "vedoucí")
            {
                prihlasitSeNaAkci.Uzivatel = uzivatelRepository.UzivatelPodleId(prihlasenyUserId);
            }
            //zjisteni clenu kteri se mohou odhlasit z akce
            if (VratRoli() == "rodič")
            {
                prihlasitSeNaAkci.ClenKOdhlaseni = clenRepository.ClenPodleId(clenId);
            }
            return View(prihlasitSeNaAkci);
        }

        [HttpPost]
        public IActionResult OdhlasitSeZAkce(PrihlasitSeNaAkciViewModel prihlasitSeNaAkci)
        {
            //odhlaseni z akce uzivatele
            if (VratRoli() == "člen" || VratRoli() == "admin" || VratRoli() == "vedoucí")
            {
                foreach (var p in prihlaseniNaAkciUzivatelRepository.VsechnaPrihlaseniNaAkci())
                {
                    if (prihlasitSeNaAkci.Akce.AkceId == p.AkceId && prihlasitSeNaAkci.Uzivatel.Id == p.UzivatelId)
                    {
                        prihlaseniNaAkciUzivatelRepository.OdebraniPrihlaseni(p);
                        break;
                    }
                }
            }
            //odhlaseni z akce clena
            if (VratRoli() == "rodič")
            {
                foreach (var p in prihlaseniNaAkciClenRepository.VsechnaPrihlaseniNaAkci())
                {
                    if (prihlasitSeNaAkci.Akce.AkceId == p.AkceId && prihlasitSeNaAkci.ClenKOdhlaseni.ClenId == p.ClenId)
                    {
                        prihlaseniNaAkciClenRepository.OdebraniPrihlaseni(p);

                        break;
                    }
                }
            }

            unitOfWork.Commit();
            TempData["Zprava"] = "Odhlášení proběhlo úspěšně.";

            return RedirectToAction("SeznamAkci");
        }
        //metoda pro sledovani clenu/uzivatelu, kteri byli odhlaseni z akce z duvodu editace
        public void Ovlivneni(int druzinaId, bool nova, bool puvodni, Guid akceId)
        {
            if (!nova && puvodni)
            {
                foreach (var p in prihlaseniNaAkciClenRepository.VsechnaPrihlaseniNaAkci())
                {
                    if (p.AkceId == akceId && clenRepository.VsichniClenove().Any(x => x.ClenId == p.ClenId && x.DruzinaId == druzinaId))
                    {
                        ovlivneniClenove.Add(clenRepository.ClenPodleId(p.ClenId));
                        prihlaseniNaAkciClenRepository.OdebraniPrihlaseni(p);
                    }
                }
                foreach (var p in prihlaseniNaAkciUzivatelRepository.VsechnaPrihlaseniNaAkci())
                {
                    if (p.AkceId == akceId && uzivatelRepository.VsichniUzivatele().Any(x => x.Id == p.UzivatelId && x.DruzinaId == druzinaId))
                    {
                        ovlivneniUzivatele.Add(uzivatelRepository.UzivatelPodleId(p.UzivatelId));
                        prihlaseniNaAkciUzivatelRepository.OdebraniPrihlaseni(p);
                    }
                }
            }
        }

        public IActionResult PridaniAkce()
        {
            var editaceAkceViewModel = new EditaceAkceViewModel();
            if (MaPravo())
            {
                return View(editaceAkceViewModel);
            }
            return View("PristupZakazan");
        }

        [HttpPost]
        public IActionResult PridaniAkce(EditaceAkceViewModel editaceAkce)
        {
            if (!ModelState.IsValid)
            {
                return View();
            }
            else if (!KontrolaExistenceData(editaceAkce.Akce.MesicKonani, editaceAkce.Akce.DenKonani, editaceAkce.Akce.RokKonani))
            {
                TempData["Zprava"] = "Zadané datum neexistuje";
                return View();
            }
            else if (!KontrolaAktualnihoData(editaceAkce.Akce.MesicKonani, editaceAkce.Akce.DenKonani, editaceAkce.Akce.RokKonani))
            {
                TempData["Zprava"] = "Nelze zadat již proběhlé datum ani dnešní den";
                return View();
            }

            editaceAkce.Akce.AutorId = Guid.Parse(userManager.GetUserId(User));

            akceRepository.PridatAkci(editaceAkce.Akce);

            TempData["Zprava"] = "Akce úspěšně přidána";
            PridaniPovoleneDruziny(editaceAkce.Modrasci, 1, editaceAkce.Akce.AkceId);
            PridaniPovoleneDruziny(editaceAkce.Fialky, 2, editaceAkce.Akce.AkceId);
            PridaniPovoleneDruziny(editaceAkce.Delfini, 3, editaceAkce.Akce.AkceId);
            PridaniPovoleneDruziny(editaceAkce.Chobotnice, 4, editaceAkce.Akce.AkceId);
            unitOfWork.Commit();

            return RedirectToAction("SeznamAkci");
        }

        public IActionResult PrihlasitSeNaAkci(Guid id)
        {
            Archivace();
            if (!akceRepository.VsechnyAkce().Any(x => x.AkceId == id))
            {
                return View("StrankaNenalezena");
            }
            var prihlasitSeNaAkci = new PrihlasitSeNaAkciViewModel()
            {
                Akce = akceRepository.AkcePodleId(id),
                PovoleneDruzinyNaAkci = new List<PovolenaDruzina>(),
                ClenoveUzivatele = new List<Clen>(),
                ClenoveKPrihlaseni = new List<Clen>(),
                PrihlaseniNaAkciClen = prihlaseniNaAkciClenRepository.VsechnaPrihlaseniNaAkci().ToList(),
                PrihlaseniNaAkciUzivatel = prihlaseniNaAkciUzivatelRepository.VsechnaPrihlaseniNaAkci().ToList()
            };
            if (VratRoli() == "člen" || VratRoli() == "admin" || VratRoli() == "vedoucí")
            {
                Guid prihlasenyUserId = Guid.Parse(userManager.GetUserId(User));
                prihlasitSeNaAkci.Uzivatel = uzivatelRepository.UzivatelPodleId(prihlasenyUserId);
            }
            //zjisteni clenu, ktere ma rodic ve sve sprave
            if (VratRoli() == "rodič" && clenRepository.VsichniClenove() != null)
            {
                foreach (var c in clenRepository.VsichniClenove())
                {
                    if (c.SpravceId == Guid.Parse(userManager.GetUserId(User)))
                    {
                        prihlasitSeNaAkci.ClenoveUzivatele.Add(c);
                    }
                }
            }
            //zjisteni druzin, pro ktere je akce urcena
            foreach (var p in povolenaDruzinaRepository.VsechnyPovoleneDruziny())
            {
                if (p.AkceId == prihlasitSeNaAkci.Akce.AkceId)
                {
                    prihlasitSeNaAkci.PovoleneDruzinyNaAkci.Add(p);
                }
            }
            return View(prihlasitSeNaAkci);
        }

        [HttpPost]
        public IActionResult PrihlasitSeNaAkci(PrihlasitSeNaAkciViewModel prihlasitSeNaAkci)
        {
            if (VratRoli() == "člen" || VratRoli() == "admin" || VratRoli() == "vedoucí")
            {
                var prihlaseniNaAkci = new PrihlaseniNaAkciUzivatel()
                {
                    AkceId = prihlasitSeNaAkci.Akce.AkceId,
                    UzivatelId = prihlasitSeNaAkci.Uzivatel.Id,
                };

                prihlaseniNaAkciUzivatelRepository.PridaniPrihlaseni(prihlaseniNaAkci);
            }
            else
            {
                //prihlaseni vsech clenu
                foreach (var c in prihlasitSeNaAkci.ClenoveKPrihlaseni)
                {
                    if (c.Prihlasen)
                    {
                        var prihlaseniNaAkciClen = new PrihlaseniNaAkciClen()
                        {
                            AkceId = prihlasitSeNaAkci.Akce.AkceId,
                            ClenId = c.ClenId,
                        };

                        prihlaseniNaAkciClenRepository.PridaniPrihlaseni(prihlaseniNaAkciClen);
                    }
                }
            }
            unitOfWork.Commit();
            TempData["Zprava"] = "Přihlášení proběhlo úspěšně.";

            return RedirectToAction("SeznamAkci");
        }

        public IActionResult SeznamAkci()
        {
            var akceViewModel = new AkceViewModel();

            Archivace();
            //akce podle data
            akceViewModel.Akce = akceRepository.VsechnyAkce().
                OrderBy(ak => ak.RokKonani)
                .ThenBy(ak => int.Parse(DatumPrevody.MesicCiselne(ak.MesicKonani)))
                .ThenBy(ak => ak.DenKonani)
                .ToList();

            if (akceViewModel.Akce.Count() == 0)
            {
                TempData["Zprava"] = "Žádné akce k zobrazení.";
            }

            return View(akceViewModel);
        }
        //spusteno pri filtraci akci
        [HttpPost]
        public IActionResult SeznamAkci(AkceViewModel akceViewModelPost)
        {
            string mesicKHledani, nazevAkceHledani, rokKHledani;
            //prevedeni parametru na velka pismena, aby hledani nebylo case sensitive
            //pokud nebyl parametr vybran k filtraci, oznaci se jako null
            if (akceViewModelPost.NazevAkceHledani != null)
            {
                nazevAkceHledani = akceViewModelPost.NazevAkceHledani.ToUpper();
            }
            else
            {
                nazevAkceHledani = null;
            }

            if (akceViewModelPost.MesicKHledani != null)
            {
                mesicKHledani = akceViewModelPost.MesicKHledani.ToUpper();
            }
            else
            {
                mesicKHledani = null;
            }

            if (akceViewModelPost.RokKHledani != null)
            {
                rokKHledani = akceViewModelPost.RokKHledani.ToUpper();
            }
            else
            {
                rokKHledani = null;
            }

            var vsechnyAkce = akceRepository.VsechnyAkce();

            //filtrace akci
            IEnumerable<Akce> akce = from a in vsechnyAkce
                                     where
                                     (
                                     (string.IsNullOrEmpty(nazevAkceHledani) || a.Nazev.ToUpper().Contains(nazevAkceHledani))
                                     &&
                                     (a.MesicKonani.ToUpper() == mesicKHledani || string.IsNullOrEmpty(mesicKHledani))
                                     &&
                                     (a.RokKonani.ToString() == rokKHledani || string.IsNullOrEmpty(rokKHledani))
                                     )
                                     orderby a.Nazev
                                     select a;

            var akceViewModel = new AkceViewModel();

            //zobrazeni vyfiltrovanych akci podle data
            akceViewModel.Akce = akce.OrderBy(ak => ak.RokKonani)
                                     .ThenBy(ak => int.Parse(DatumPrevody.MesicCiselne(ak.MesicKonani)))
                                     .ThenBy(ak => ak.DenKonani).ToList();

            if (akceViewModel.Akce.Count == 0)
            {
                TempData["Zprava"] = "Žádné akce k zobrazení.";
            }

            return View(akceViewModel);
        }

        public string VratRoli()
        {
            Guid prihlasenyUserId = Guid.Parse(userManager.GetUserId(User));
            int prihlasenyRoleId = uzivatelRepository.UzivatelPodleId(prihlasenyUserId).RoleId;
            string prihlasenyRole = roleRepository.RolePodleId(prihlasenyRoleId).Nazev;
            return prihlasenyRole;
        }

        private bool KontrolaAktualnihoData(string mesic, int? den, int? rok)
        {
            if
                (
                (rok < int.Parse(DatumPrevody.VratRok())) ||
                (rok == int.Parse(DatumPrevody.VratRok()) && int.Parse(DatumPrevody.MesicCiselne(mesic)) < DatumPrevody.VratMesic()) ||
                (rok == int.Parse(DatumPrevody.VratRok()) && int.Parse(DatumPrevody.MesicCiselne(mesic)) == DatumPrevody.VratMesic() && den <= int.Parse(DatumPrevody.VratDen()))
                )
            {
                return false;
            }
            else return true;
        }

        private bool KontrolaExistenceData(string mesic, int? den, int? rok)
        {
            if (
                (
                mesic == "únor" && PrestupnyRok(rok) && den >= 30)
                || (mesic == "únor" && !PrestupnyRok(rok) && den >= 29)
                || (mesic == "duben" && den >= 31)
                || (mesic == "červen" && den >= 31)
                || (mesic == "září" && den >= 31)
                || (mesic == "listopad" && den >= 31)
                )
            {
                return false;
            }
            else return true;
        }

        private bool PrestupnyRok(int? rok)
        {
            if (rok % 4 == 0)
            {
                return true;
            }
            else return false;
        }

        private void PridaniPovoleneDruziny(bool druzina, int druzinaId, Guid akceId)
        {
            if (druzina)
            {
                var povolenaDruzina = new PovolenaDruzina()
                {
                    AkceId = akceId,
                    DruzinaId = druzinaId
                };
                povolenaDruzinaRepository.PridaniPovoleneDruziny(povolenaDruzina);
            }
        }

        private static async Task OvlivneniMail(Uzivatel komu, string nazevAkce, string jmeno = null)
        {
            var apiKey = "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";
            var client = new SendGridClient(apiKey);
            var from = new EmailAddress("modrinkytabor@seznam.cz");
            var to = new EmailAddress(komu.Email);
            var subject = "Odhlášení z akce " + nazevAkce;
            var plainTextContent = string.Empty;
            if (jmeno == null)
            {
                plainTextContent = "Dobrý den, kvůli změně družin, které se mohou zůčastnit akce " + nazevAkce + ", jste byla z této akce odhlášena.";
            }
            else
            {
                plainTextContent = "Dobrý den, kvůli změně družin, které se mohou zůčastnit akce " + nazevAkce + ", byl člen "
              + jmeno + " z této akce odhlášen.";
            }

            var msg = MailHelper.CreateSingleEmail
                (
                from, to, subject, plainTextContent, null
                );
            var response = await client.SendEmailAsync(msg);
        }
    }
}