﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using ModrinkyWebApp.Models;
using ModrinkyWebApp.Models.Repositare;
using System;
using System.Linq;
using X.PagedList;

namespace Modrinky.Controllers
{
    public class AktualityController : Controller
    {
        private readonly IAktualitaRepository aktualitaRepository;
        private readonly IUzivatelRepository uzivatelRepository;
        private readonly IRoleRepository roleRepository;
        private readonly UserManager<Uzivatel> userManager;
        private readonly IUnitOfWork unitOfWork;

        public AktualityController(IAktualitaRepository aktualitaRepository,
            UserManager<Uzivatel> userManager,
            IUnitOfWork unitOfWork
            )
        {
            this.aktualitaRepository = unitOfWork.AktualitaRepository;
            this.userManager = userManager;
            this.unitOfWork = unitOfWork;
            this.uzivatelRepository = unitOfWork.UzivatelRepository;
            this.roleRepository = unitOfWork.RoleRepository;
        }

        public string VratRoli()
        {
            Guid prihlasenyUserId = Guid.Parse(userManager.GetUserId(User));
            int prihlasenyRoleId = uzivatelRepository.VsichniUzivatele().FirstOrDefault(x => x.Id == prihlasenyUserId).RoleId;
            string prihlasenyRole = roleRepository.VsechnyRole().FirstOrDefault(x => x.RoleId == prihlasenyRoleId).Nazev;
            return prihlasenyRole;
        }

        public bool MaPravo()
        {
            if (VratRoli() == "vedoucí" || VratRoli() == "admin") return true;
            else return false;
        }

        public object SeznamAktualit(int? pageNumber)
        {
            //pro rozdeleni aktualit do stranek
            var temp = aktualitaRepository.VsechnyAktuality().
                OrderByDescending(ak => ak.Datum).ToList();
            var page = pageNumber ?? 1;
            if (temp.Count() == 0)
            {
                TempData["Zprava"] = "Žádné aktuality k zobrazení.";
            }
            //na jedne strance bude zobrazeno pet aktualit
            var onePageOfProducts = temp.ToPagedList(page, 5);
            ViewBag.OnePageOfProducts = onePageOfProducts;
            return View();
        }

        [Authorize]
        public IActionResult PridaniAktuality()
        {
            if (VratRoli() == "vedoucí" || VratRoli() == "admin")
            {
                return View();
            }
            return View("PristupZakazan");
        }

        [Authorize]
        [HttpPost]
        public IActionResult PridaniAktuality(Aktualita aktualita)
        {
            //zadani bylo bezchybne
            if (!ModelState.IsValid)
            {
                return View();
            }
            aktualita.AutorId = Guid.Parse(userManager.GetUserId(User));

            aktualitaRepository.PridatAktualitu(aktualita);
            unitOfWork.Commit();
            TempData["Zprava"] = "Aktualita úspěšně přidána.";
            return RedirectToAction("SeznamAktualit");
        }

        [Authorize]
        public IActionResult OdebraniAktuality(Guid id)
        {
            if (!MaPravo())
            {
                return View("PristupZakazan");
            }
            else if (!aktualitaRepository.VsechnyAktuality().Any(x => x.AktualitaId == id))
            {
                return View("StrankaNenalezena");
            }
            //vedouci muze odebrat aktualitu pouze je-li jejim autorem
            else if (VratRoli() == "vedoucí" &&
                aktualitaRepository.AktualitaPodleId(id).AutorId != Guid.Parse(userManager.GetUserId(User)))
            {
                return View("PristupZakazan");
            }
            return View();
        }

        [Authorize]
        [HttpPost]
        public IActionResult OdebraniAktuality(Guid id, bool onPost = true)
        {
            aktualitaRepository.OdebratAktuality(id);
            unitOfWork.Commit();
            TempData["Zprava"] = "Aktualita úspěšně odebrána.";

            return RedirectToAction("SeznamAktualit");
        }

        [Authorize]
        public IActionResult EditaceAktuality(Guid id)
        {
            if (!MaPravo())
            {
                return View("PristupZakazan");
            }
            else if (!aktualitaRepository.VsechnyAktuality().Any(x => x.AktualitaId == id))
            {
                return View("StrankaNenalezena");
            }
            //vedouci muze upravit aktualitu pouze je-li jejim autorem
            else if (VratRoli() == "vedoucí" &&
                aktualitaRepository.AktualitaPodleId(id).AutorId != Guid.Parse(userManager.GetUserId(User)))
            {
                return View("PristupZakazan");
            }
            Aktualita a = aktualitaRepository.AktualitaPodleId(id);
            //pro editaci je pouzito dvou formularu, aby nedochazelo k validaci jiz pri prvotnim zadani
            return View("EditaceAktuality2", a); 
        }

        [Authorize]
        [HttpPost]
        public IActionResult EditaceAktuality(Aktualita aktualita, bool onPost = true)
        {
            if (ModelState.IsValid)
            {
                aktualitaRepository.EditaceAktuality(aktualita);
                unitOfWork.Commit();
                TempData["Zprava"] = "Aktualita úspěšně upravena.";

                return RedirectToAction("SeznamAktualit");
            }
            else
            {
                return View("EditaceAktuality");
            }
        }
    }
}