﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using ModrinkyWebApp.Models;
using ModrinkyWebApp.Models.Repositare;
using ModrinkyWebApp.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;

namespace ModrinkyWebApp.Controllers
{
    [Authorize]
    public class ArchivAkciController : Controller
    {
        private readonly IAkceRepository akceRepository;
        private readonly IUzivatelRepository uzivatelRepository;
        private readonly IClenRepository clenRepository;
        private readonly UserManager<Uzivatel> userManager;
        private readonly IPrihlaseniArchivRepository prihlaseniArchivRepository;
        private readonly IClenArchivRepository clenArchivRepository;
        private readonly IRoleRepository roleRepository;
        private readonly IUnitOfWork unitOfWork;

        public ArchivAkciController(IAkceRepository akceRepository,
            UserManager<Uzivatel> userManager,

            IUnitOfWork unitOfWork

            )
        {
            this.akceRepository = unitOfWork.AkceRepository;
            this.userManager = userManager;
            this.unitOfWork = unitOfWork;
            this.uzivatelRepository = unitOfWork.UzivatelRepository;
            this.clenRepository = unitOfWork.ClenRepository;
            this.prihlaseniArchivRepository = unitOfWork.PrihlaseniArchivRepository;
            this.clenArchivRepository = unitOfWork.ClenArchivRepository;
            this.roleRepository = unitOfWork.RoleRepository;
        }

        public string VratRoli()
        {
            Guid prihlasenyUserId = Guid.Parse(userManager.GetUserId(User));
            int prihlasenyRoleId = uzivatelRepository.UzivatelPodleId(prihlasenyUserId).RoleId;
            string prihlasenyRole = roleRepository.RolePodleId(prihlasenyRoleId).Nazev;
            return prihlasenyRole;
        }

        public bool MaPravo()
        {
            if (VratRoli() == "vedoucí" || VratRoli() == "admin")
                return true;
            else return false;
        }

        public IActionResult SeznamArchivnichAkci()
        {
            //seznam archivnich akci je pristupny pouze vedoucim a adminum
            if (MaPravo())
            {
                if (akceRepository.VsechnyArchivniAkce().ToList().Count() > 0)
                {
                    var akceArchiv = new AkceArchivViewModel()
                    {
                        ArchivAkci = new List<Akce>()
                    };
                    //serazeni archivnich akci podle data
                    akceArchiv.ArchivAkci = akceRepository.VsechnyArchivniAkce().
                 OrderByDescending(ak => ak.RokKonani).ThenByDescending(ak => int.Parse(DatumPrevody.MesicCiselne(ak.MesicKonani))).ThenByDescending(ak => ak.DenKonani).ToList();
                    return View(akceArchiv);
                }
                else
                {
                    return View("ZadnyObsahKZobrazeni");
                }
            }
            else
            {
                return View("PristupZakazan");
            }
        }

        public IActionResult DetailArchivniAkce(Guid Id)
        {
            if (MaPravo())
            {
                var akceArchiv = new AkceArchivViewModel()
                {
                    Akce = akceRepository.VsechnyArchivniAkce().FirstOrDefault(a => a.AkceId == Id),
                    ArchivClenu = new List<ClenArchiv>(),
                    ArchivniPrihlasen = new List<PrihlaseniArchiv>()
                };
                if (akceArchiv.Akce == null)
                {
                    return View("ZadnyObsahKZobrazeni");
                }
                else
                {   //v detailu akce jsou zobrazeni i clenove a uzivatele, kteri se akce zucastnili (bez ohledu na to, zda maji stale aktivni ucet v aplikaci)
                    foreach (var p in prihlaseniArchivRepository.VsechnaArchivniPrihlaseni())
                    {
                        if ((p.AkceArchivId == akceArchiv.Akce.AkceId) && akceArchiv.Akce.ArchivniAkce)
                        {
                            akceArchiv.ArchivniPrihlasen.Add(p);
                        }
                    }
                    foreach (var p in akceArchiv.ArchivniPrihlasen)
                    {
                        foreach (var c in clenArchivRepository.VsichniArchivniCleni())
                        {
                            if (c.ClenArchivId == p.ClenArchivId)
                            {
                                akceArchiv.ArchivClenu.Add(c);
                            }
                        }
                    }

                    return View(akceArchiv);
                }
            }
            else return View("PristupZakazan");
        }
    }
}