﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using ModrinkyWebApp.Models;
using ModrinkyWebApp.Models.Repositare;
using ModrinkyWebApp.ViewModels;
using SendGrid;
using SendGrid.Helpers.Mail;

namespace ModrinkyWebApp.Controllers
{
    [Authorize]
    public class MojeDruzinaController : Controller
    {
        private static string ApiKey { get; set; } = "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";
        private readonly IUzivatelRepository uzivatelRepository;
        private readonly IDruzinaRepository druzinaRepository;
        private readonly UserManager<Uzivatel> userManager;
        private readonly IRoleRepository roleRepository;
        private readonly IClenRepository clenRepository;
        private readonly IAkceRepository akceRepository;
        private readonly IPovolenaDruzinaRepository povolenaDruzinaRepository;
        private readonly IUnitOfWork unitOfWork;


        public MojeDruzinaController(IUnitOfWork unitOfWork,
            UserManager<Uzivatel> userManager)
            
        {
            this.uzivatelRepository = unitOfWork.UzivatelRepository;
            this.druzinaRepository = unitOfWork.DruzinaRepository;
            this.userManager = userManager;
            this.roleRepository = unitOfWork.RoleRepository;
            this.clenRepository = unitOfWork.ClenRepository;
            this.akceRepository = unitOfWork.AkceRepository;
            this.povolenaDruzinaRepository = unitOfWork.PovolenaDruzinaRepository;
            this.unitOfWork = unitOfWork;
        }
        public string VratRoli()
        {
            Guid prihlasenyUserId = Guid.Parse(userManager.GetUserId(User));
            int prihlasenyRoleId = uzivatelRepository.VsichniUzivatele().FirstOrDefault(x => x.Id == prihlasenyUserId).RoleId;
            string prihlasenyRole = roleRepository.VsechnyRole().FirstOrDefault(x => x.RoleId == prihlasenyRoleId).Nazev;
            return prihlasenyRole;
        }
        public IActionResult DetailDruziny(Guid id)
        {
            Guid prihlasenyUserId = Guid.Parse(userManager.GetUserId(User));
            Uzivatel prihlaseny = uzivatelRepository.UzivatelPodleId(prihlasenyUserId);
            int? druzinaId = prihlaseny.DruzinaId;

            MojeDruzinaViewModel mojeDruzina = new MojeDruzinaViewModel();

            if(VratRoli() == "vedoucí" || VratRoli() == "admin")
            {
                return View("PristupZakazan");
            }
           
            if(
                (VratRoli() == "člen" && druzinaId == null) 
                ||
                (VratRoli() == "rodič" && !clenRepository.ClenoveUzivatele(prihlasenyUserId).Any())
              )
            {
                mojeDruzina.MaDruzinu = false;
                return View(mojeDruzina);
            }
            else
            {
                mojeDruzina.MaDruzinu = true;
                mojeDruzina.UzivateleVDruzine = new List<Uzivatel>();
                mojeDruzina.ClenoveVDruzine = new List<Clen>();
                mojeDruzina.AkceDruziny = new List<Akce>();

                if(VratRoli() == "rodič")
                {
                    //zjisteni druziny clena
                    mojeDruzina.Clen = clenRepository.ClenPodleId(id);
                    mojeDruzina.Druzina = druzinaRepository.VsechnyDruziny()
                                                           .FirstOrDefault(x => x.DruzinaId == mojeDruzina.Clen.DruzinaId);

                    if(mojeDruzina.Druzina == null)
                    {
                        mojeDruzina.MaDruzinu = false;
                        return View(mojeDruzina);
                    }
                }
                else if(VratRoli() == "člen")
                {
                    mojeDruzina.Druzina = druzinaRepository.VsechnyDruziny().FirstOrDefault(x => x.DruzinaId == druzinaId);
                }

                mojeDruzina.VedouciDruziny = uzivatelRepository.UzivatelPodleId(mojeDruzina.Druzina.VedouciDruzinyId);

                //uzivatele druziny
                foreach(var u in uzivatelRepository.VsichniUzivatele())
                {
                    if(u.DruzinaId == mojeDruzina.Druzina.DruzinaId)
                    {
                        mojeDruzina.UzivateleVDruzine.Add(u);
                    }
                }

                //clenove druziny
                foreach(var c in clenRepository.VsichniClenove())
                {
                    if(c.DruzinaId == mojeDruzina.Druzina.DruzinaId)
                    {
                        mojeDruzina.ClenoveVDruzine.Add(c);
                    }
                }
                
                //aby byly zobrazeny jen ty akce, kterych se skutecne druzina muze zucastnit
                foreach(var p in povolenaDruzinaRepository.VsechnyPovoleneDruziny())
                {
                    if(p.DruzinaId == mojeDruzina.Druzina.DruzinaId)
                    {
                        Akce akce = akceRepository.AkcePodleId(p.AkceId);
                        if (!akce.ArchivniAkce)
                        {
                            mojeDruzina.AkceDruziny.Add(akce);
                        }
                    }
                }
            }
            return View(mojeDruzina);
        }

        [HttpPost]
        public IActionResult DetailDruziny(MojeDruzinaViewModel mojeDruzina)
        {
            Guid prihlasenyUserId = Guid.Parse(userManager.GetUserId(User));
            Uzivatel prihlaseny = uzivatelRepository.UzivatelPodleId(prihlasenyUserId);
            Uzivatel vedouciDruziny = uzivatelRepository.UzivatelPodleId(mojeDruzina.VedouciDruziny.Id);

            EmailVedouci(prihlaseny, vedouciDruziny, mojeDruzina.TextEmailu).Wait();

            TempData["Zprava"] = "Zpráva úspěšně odeslána";

            return RedirectToAction("DetailDruziny", mojeDruzina.Clen.ClenId);
        }

        public IActionResult PridaniVedouciDruzine()
        {
            PridaniVedouciDruzineViewModel pridaniVedouciDruzine = new PridaniVedouciDruzineViewModel()
            {
                SeznamVedoucich = uzivatelRepository.VsichniUzivatele().Where(x => (x.RoleId != 2)).ToList(),
                Chobotnice = druzinaRepository.VsechnyDruziny().FirstOrDefault(x => x.DruzinaId == 4),
                Delfini = druzinaRepository.VsechnyDruziny().FirstOrDefault(x => x.DruzinaId == 3),

                Fialky = druzinaRepository.VsechnyDruziny().FirstOrDefault(x => x.DruzinaId == 2),
                Modrasci = druzinaRepository.VsechnyDruziny().FirstOrDefault(x => x.DruzinaId == 1)

            };
            return View(pridaniVedouciDruzine);
        }
        [HttpPost]
        public IActionResult PridaniVedouciDruzine(PridaniVedouciDruzineViewModel pridaniVedouciDruzine)
        {
            //prirazeni nove vedouci ci odebrani vedouci pro jednotlive druziny
            Druzina modrasciZmena = druzinaRepository.VsechnyDruziny().FirstOrDefault(x => x.DruzinaId == 1);
      
            if (pridaniVedouciDruzine.NovyVedouciModrasci == Guid.Empty)
            {
                modrasciZmena.VedouciDruzinyId = null;
            }
            else
            {
                modrasciZmena.VedouciDruzinyId = pridaniVedouciDruzine.NovyVedouciModrasci;

            }
            druzinaRepository.UpravaDruziny(modrasciZmena);

            Druzina fialkyZmena = druzinaRepository.VsechnyDruziny().FirstOrDefault(x => x.DruzinaId == 2);
            if (pridaniVedouciDruzine.NovyVedouciFialky == Guid.Empty)
            {
                fialkyZmena.VedouciDruzinyId = null;
            }
            else
            {
                fialkyZmena.VedouciDruzinyId = pridaniVedouciDruzine.NovyVedouciFialky;

            }
            druzinaRepository.UpravaDruziny(fialkyZmena);

            Druzina delfiniZmena = druzinaRepository.VsechnyDruziny().FirstOrDefault(x => x.DruzinaId == 3);
            if (pridaniVedouciDruzine.NovyVedouciDelfini == Guid.Empty)
            {
                delfiniZmena.VedouciDruzinyId = null;
            }
            else
            {
                delfiniZmena.VedouciDruzinyId = pridaniVedouciDruzine.NovyVedouciDelfini;

            }
            druzinaRepository.UpravaDruziny(delfiniZmena);

            Druzina chobotniceZmena = druzinaRepository.VsechnyDruziny().FirstOrDefault(x => x.DruzinaId == 4);
            if (pridaniVedouciDruzine.NovyVedouciChobotnice == Guid.Empty)
            {
                chobotniceZmena.VedouciDruzinyId = null;
            }
            else
            {
                chobotniceZmena.VedouciDruzinyId = pridaniVedouciDruzine.NovyVedouciChobotnice;

            }
            druzinaRepository.UpravaDruziny(chobotniceZmena);

            try
            {
                unitOfWork.Commit();
            }
            catch
            {
                TempData["Zprava"] = "Tato akce není povolena, každý úživatel může být voudoucí právě jedné druižny.";
                return RedirectToAction("PridaniVedouciDruzine");

            }

            return RedirectToAction("PridaniVedouciDruzine");
        }
        static async Task EmailVedouci(Uzivatel odesilatel, Uzivatel vedouciDruziny, string text)
        {
            var klient = new SendGridClient(ApiKey);
            var od = new EmailAddress(odesilatel.Email);
            var pro = new EmailAddress(vedouciDruziny.Email);
            var predmet = "Zpráva ze stránek 9. oddílu Modřinky";

            var msg = MailHelper.CreateSingleEmail
                (
                od, pro, predmet, text, null
                );

            await klient.SendEmailAsync(msg);
        }
    }
}