﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using ModrinkyWebApp.Models;
using ModrinkyWebApp.Models.Repositare;
using ModrinkyWebApp.ViewModels;
using System;
using System.Linq;

namespace ModrinkyWebApp.Controllers
{
    public class UserController : Controller
    {
        private readonly IUzivatelRepository uzivatelRepository;
        private readonly UserManager<Uzivatel> userManager;
        private readonly IRoleRepository roleRepository;
        private readonly IDruzinaRepository druzinaRepository;
        private readonly IAkceRepository akceRepository;
        private readonly IPrihlaseniNaAkciUzivatelRepository prihlaseniNaAkciUzivatelRepository;
        private readonly IPrihlaseniNaAkciClenRepository prihlaseniNaAkciClenRepository;
        private readonly IClenRepository clenRepository;

        public UserController(UserManager<Uzivatel> userManager,
            IUzivatelRepository uzivatelRepository,
            IPrihlaseniNaAkciUzivatelRepository prihlaseniNaAkciUzivatelRepository,
            IPrihlaseniNaAkciClenRepository prihlaseniNaAkciClenRepository,
            IDruzinaRepository druzinaRepository,
            IAkceRepository akceRepository,
            IClenRepository clenRepository,
            IRoleRepository roleRepository)
        {
            this.userManager = userManager;
            this.roleRepository = roleRepository;
            this.uzivatelRepository = uzivatelRepository;
            this.druzinaRepository = druzinaRepository;
            this.akceRepository = akceRepository;
            this.prihlaseniNaAkciClenRepository = prihlaseniNaAkciClenRepository;
            this.prihlaseniNaAkciUzivatelRepository = prihlaseniNaAkciUzivatelRepository;
            this.clenRepository = clenRepository;
        }

        [Authorize]
        //zobrazeni informaci o profilu uzivatele
        public IActionResult Index()
        {
            Guid prihlasenyUserId = Guid.Parse(userManager.GetUserId(User));
            int roleId = uzivatelRepository.UzivatelPodleId(prihlasenyUserId).RoleId;

            var user = new UserViewModel()
            {
                Druziny = druzinaRepository.VsechnyDruziny().ToList(),
                Akce = akceRepository.VsechnyAkce().ToList(),
                PrihlaseniNaAkciUzivatel = prihlaseniNaAkciUzivatelRepository.PrihlaseniNaAkciUzivatele(prihlasenyUserId).ToList(),
                Clenove = clenRepository.ClenoveUzivatele(prihlasenyUserId).ToList(),
                Role = roleRepository.RolePodleId(roleId)
            };

            user.Uzivatel = uzivatelRepository.UzivatelPodleId(prihlasenyUserId);
            return View(user);
        }

        //rodic si muze rozkliknout detail clena v jeho sprave
        public IActionResult DetailClena(Guid id)
        {
            var detailClena = new DetailClenaViewModel()
            {
                Clen = clenRepository.ClenPodleId(id),
                Druziny = druzinaRepository.VsechnyDruziny().ToList(),
                PrihlaseniNaAkciClen = prihlaseniNaAkciClenRepository.PrihlaseniNaAkciClen(id).ToList(),
                Akce = akceRepository.VsechnyAkce().ToList()
            };
            return View(detailClena);
        }

    }
}