﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using ModrinkyWebApp.ViewModels;
using ModrinkyWebApp.Models;
using ModrinkyWebApp.Models.Repositare;
using Microsoft.AspNetCore.Authorization;

namespace ModrinkyWebApp.Controllers
{
    [Authorize]
    public class UzivateleController : Controller
    {
        private readonly IUzivatelRepository uzivatelRepository;
        private readonly IUnitOfWork unitOfWork;
        private readonly IClenRepository clenRepository;
        private readonly UserManager<Uzivatel> userManager;
        
        private readonly IDruzinaRepository druzinaRepository;
        private readonly IClenArchivRepository clenArchivRepository;
        private readonly IPrihlaseniNaAkciClenRepository prihlaseniNaAkciClenRepository;
        private readonly IPrihlaseniNaAkciUzivatelRepository prihlaseniNaAkciUzivatelRepository;
        private readonly IRoleRepository roleRepository;


        public UzivateleController(IUzivatelRepository uzivatelRepository,
            UserManager<Uzivatel> userManager,
            IUnitOfWork unitOfWork
            )
        {
            this.uzivatelRepository = unitOfWork.UzivatelRepository;
            this.userManager = userManager;
            this.unitOfWork = unitOfWork;
            this.clenRepository = unitOfWork.ClenRepository;
            this.druzinaRepository = unitOfWork.DruzinaRepository;
            this.clenArchivRepository = unitOfWork.ClenArchivRepository;
            this.prihlaseniNaAkciUzivatelRepository = unitOfWork.PrihlaseniNaAkciUzivatelRepository;
            this.prihlaseniNaAkciClenRepository = unitOfWork.PrihlaseniNaAkciClenRepository;
            this.roleRepository = unitOfWork.RoleRepository;
        }
        public string VratRoli()
        {
            Guid prihlasenyUserId = Guid.Parse(userManager.GetUserId(User));
            int prihlasenyRoleId = uzivatelRepository.UzivatelPodleId(prihlasenyUserId).RoleId;
            string prihlasenyRole = roleRepository.RolePodleId(prihlasenyRoleId).Nazev;
            return prihlasenyRole;
        }
        public bool MaPravo()
        {
            if (VratRoli() == "vedoucí" || VratRoli() == "admin")
                return true;
            else return false;
        }
        public IActionResult SeznamUzivatelu()
        {
            if (!MaPravo())
            {
                return View("PristupZakazan");
            }

            var uzivateleViewModel = new UzivateleViewModel()
            {
                Nadpis = "Uzivatele",
            };
            //seznam uzivatelu je setrizen podle abecedy
            uzivateleViewModel.Uzivatele = uzivatelRepository.VsichniUzivatele().OrderBy(u => u.Prijmeni).ThenBy(u => u.KrestniJmeno).ToList();
            return View(uzivateleViewModel);
        }

        //ve chvili kdy byla zvolena filtrace uzivatelu
        [HttpPost]
        public IActionResult SeznamUzivatelu(UzivateleViewModel uzivatele2)
        {
            var vsechniUzivatele = uzivatelRepository.VsichniUzivatele();
            List<Uzivatel> uzivateleSeznamList = new List<Uzivatel>();
            List<Uzivatel> uzivateleSeznamList2 = new List<Uzivatel>();
            string jmenoKHledani, prijmeniKHledani;
            //parametry filtrace jsou prevedeny na velka pismena, aby hledani nebylo case sensitive
            //ve chvili kdy parametr nebyl vyuzit k hledani, je oznacen jako null
            if (uzivatele2.JmenoKHledani != null)
            {
                jmenoKHledani = uzivatele2.JmenoKHledani.ToUpper();
            }
            else
            {
                jmenoKHledani = null;
            }

            if (uzivatele2.PrijmeniKHledani != null)
            {
                prijmeniKHledani = uzivatele2.PrijmeniKHledani.ToUpper();
            }
            else
            {
                prijmeniKHledani = null;
            }
            //provedeni filtrace
            IEnumerable<Uzivatel> uzivateleSeznam =
                    from u in vsechniUzivatele
                    where
                    (string.IsNullOrEmpty(jmenoKHledani) || u.KrestniJmeno.ToUpper().Contains(jmenoKHledani))
                    &&
                    (string.IsNullOrEmpty(prijmeniKHledani) || u.Prijmeni.ToUpper().Contains(prijmeniKHledani))
                    select u;
            var uzivatele = new UzivateleViewModel()
            {
                Nadpis = "Uzivatele",
            };
            //vyfiltrovani uzivatele jsou setrizeni podle abecedy
            uzivatele.Uzivatele = uzivateleSeznam.OrderBy(u => u.Prijmeni).ThenBy(u => u.KrestniJmeno).ToList();
            return View(uzivatele);

        }

        //detail uzivatele
        public IActionResult Detail(Guid id)
        {
            if (!MaPravo())
            {
                return View("PristupZakazan");
            }
            else if(!uzivatelRepository.VsichniUzivatele().Any(x => x.Id == id))
            {
                return View("StrankaNenalezena");
            }

            var detailUzivateleViewModel = new DetailUzivateleViewModel()
            {
                Clenove = clenRepository.VsichniClenove().ToList(),
                Uzivatel = uzivatelRepository.UzivatelPodleId(id),
                Druziny = druzinaRepository.VsechnyDruziny().ToList()
            };

            return View(detailUzivateleViewModel);
        }

        //pridani noveho clena rodicovi
        public IActionResult PridaniClena(Guid id)
        {
            if (!MaPravo())
            {
                return View("PristupZakazan");
            }
            else if (!uzivatelRepository.VsichniUzivatele().Any(x => x.Id == id))
            {
                return View("StrankaNenalezena");
            }
            PridaniClenaViewModel pridaniClenaViewModel = new PridaniClenaViewModel()
            {
                Druziny = druzinaRepository.VsechnyDruziny().ToList(),

                EditovanyUzivatel = uzivatelRepository.UzivatelPodleId(id),
                NovyClen = new Clen
                {
                    ClenId = Guid.NewGuid(),
                    KrestniJmeno = "",
                    Prijmeni = ""
                }

            };
            
                return View("PridaniClena", pridaniClenaViewModel);
        }

        [HttpPost]
        public IActionResult PridaniClena(PridaniClenaViewModel pridaniClenaViewModel)
        {
            if (ModelState.IsValid)
            {
                switch (pridaniClenaViewModel.NazevNoveDruziny)
                {
                    case "Modrásci":
                        pridaniClenaViewModel.NovyClen.DruzinaId = 1;
                        break;
                    case "Fialky":
                        pridaniClenaViewModel.NovyClen.DruzinaId = 2;
                        break;
                    case "Delfíni":
                        pridaniClenaViewModel.NovyClen.DruzinaId = 3;
                        break;
                    case "Chobotnice":
                        pridaniClenaViewModel.NovyClen.DruzinaId = 4;
                        break;


                }
                clenRepository.PridaniClena(pridaniClenaViewModel.NovyClen);
                ClenArchiv pom = new ClenArchiv()
                {
                    ClenArchivId = pridaniClenaViewModel.NovyClen.ClenId,
                    KrestniJmeno = pridaniClenaViewModel.NovyClen.KrestniJmeno,
                    Prijmeni = pridaniClenaViewModel.NovyClen.Prijmeni,
                };
                clenArchivRepository.PridaniArchivnihoClena(pom);
                unitOfWork.Commit();
                 TempData["Zprava"] = "Člen úspěšně přidán.";

                return RedirectToAction("Detail", new { @id = pridaniClenaViewModel.NovyClen.SpravceId });
            }

            return View("PridaniClena", pridaniClenaViewModel);
        }
        public IActionResult DetailClena(Guid id)
        {
            if (!MaPravo())
            {
                return View("PristupZakazan");
            }
            else if (!clenRepository.VsichniClenove().Any(x => x.ClenId == id))
            {
                return View("StrankaNenalezena");
            }
            var detailClena = new DetailClenaViewModel()
            {

                Clen = clenRepository.ClenPodleId(id),
                Druziny = druzinaRepository.VsechnyDruziny().ToList(),
            };
            detailClena.Uzivatel = uzivatelRepository.UzivatelPodleId(detailClena.Clen.SpravceId);

            return View(detailClena);
        }
        public IActionResult OdebraniUzivatele(Uzivatel uzivatel)
        {
            if (!(VratRoli() == "admin"))
            {
                return View("PristupZakazan");
            }
            else if (!uzivatelRepository.VsichniUzivatele().Any(x => x.Id == uzivatel.Id))
            {
                return View("StrankaNenalezena");
            }
            return View(uzivatel);

        }
        [HttpPost]
        public IActionResult OdebraniUzivatele(Uzivatel uzivatel, bool onPost = true)
        {
            //pred odebranim je uzivatel odhlasen ze vsech akci
            foreach (var p in prihlaseniNaAkciUzivatelRepository.VsechnaPrihlaseniNaAkci())
            {
                if (p.UzivatelId == uzivatel.Id)
                {
                    prihlaseniNaAkciUzivatelRepository.OdebraniPrihlaseni(p.PrihlaseniNaAkciUzivatelId);
                }
                
            }
            //pokud byl uzivatel vedouci druziny, je ji odebran
            foreach(var u in druzinaRepository.VsechnyDruziny())
            {
                if (u.VedouciDruzinyId == uzivatel.Id)
                {
                    u.VedouciDruzinyId = null;
                }
                break;
            }
            
            uzivatelRepository.OdebraniUzivatele(uzivatel.Id);
            unitOfWork.Commit();
            TempData["Zprava"] = "Uživatel úspěšně odebrán.";
            return RedirectToAction("SeznamUzivatelu");
        }
        public IActionResult OdebraniClena(Guid id)
        {
            if (!MaPravo())
            {
                return View("PristupZakazan");
            }
            else if (!clenRepository.VsichniClenove().Any(x => x.ClenId == id))
            {
                return View("StrankaNenalezena");
            }
            var clen = clenRepository.ClenPodleId(id);
            
                return View(clen);
            
            
        }
        [HttpPost]
        public IActionResult OdebraniClena(Clen clen, bool onPost = true)
        {
            //pred odebranim je clen odhlasen ze vsech akci
            foreach (var p in prihlaseniNaAkciClenRepository.VsechnaPrihlaseniNaAkci())
            {
                if (p.ClenId == clen.ClenId)
                {
                    prihlaseniNaAkciClenRepository.OdebraniPrihlaseni(p.PrihlaseniNaAkciClenId);
                }
            }
            Guid idSpravce = clen.SpravceId;
            clenRepository.OdebraniClena(clen);
            unitOfWork.Commit();
            TempData["Zprava"] = "Clen úspěšně odebrána.";
            return RedirectToAction("Detail", new { @id = idSpravce });
        }
        public IActionResult PridaniDruzinyUzivateli(Guid id)
        {
            if (!MaPravo())
            {
                return View("PristupZakazan");
            }
            else if (!uzivatelRepository.VsichniUzivatele().Any(x => x.Id == id))
            {
                return View("StrankaNenalezena");
            }
            var pridaniDruzinyUzivateliViewModel = new PridaniDruzinyUzivateliViewModel()
            {
                Uzivatel = uzivatelRepository.UzivatelPodleId(id),
                Druziny = druzinaRepository.VsechnyDruziny().ToList()
            };
            if (pridaniDruzinyUzivateliViewModel.Uzivatel.DruzinaId != null)
            {
                pridaniDruzinyUzivateliViewModel.IdStareDruziny = pridaniDruzinyUzivateliViewModel.Uzivatel.DruzinaId;
            }
            return View(pridaniDruzinyUzivateliViewModel);
        }
        [HttpPost]
        public IActionResult PridaniDruzinyUzivateli(PridaniDruzinyUzivateliViewModel pridaniDruzinyUzivateli)
        {
            var doplneni = uzivatelRepository.UzivatelPodleId(pridaniDruzinyUzivateli.Uzivatel.Id);
            switch (pridaniDruzinyUzivateli.NazevNoveDruziny)
            {
                case "Modrásci":
                    doplneni.DruzinaId = 1;
                    break;
                case "Fialky":
                    doplneni.DruzinaId = 2;
                    break;
                case "Delfíni":
                    doplneni.DruzinaId = 3;
                    break;
                case "Chobotnice":
                    doplneni.DruzinaId = 4;
                    break;
                default:
                    doplneni.DruzinaId = null;
                    break;
            }
            if (pridaniDruzinyUzivateli.IdStareDruziny != doplneni.DruzinaId)
            {
                foreach (var p in prihlaseniNaAkciUzivatelRepository.VsechnaPrihlaseniNaAkci())
                {
                    if (p.UzivatelId == doplneni.Id)
                    {
                        prihlaseniNaAkciUzivatelRepository.OdebraniPrihlaseni(p.PrihlaseniNaAkciUzivatelId);
                    }
                }
            }

            uzivatelRepository.EditaceUzivatele(doplneni);
            
            //v zavislosti na provedene akci je vypsana informacni hlaska
            if (doplneni.DruzinaId == null)
            {
                unitOfWork.Commit();
                TempData["Zprava"] = "Družina odebrána.";
            }
            else
            {
                unitOfWork.Commit();
                TempData["Zprava"] = "Družina přidána.";
            }
            return RedirectToAction("SeznamUzivatelu");
        }
        public IActionResult PridaniDruzinyClenovi(Guid id)
        {
            if (!MaPravo())
            {
                return View("PristupZakazan");
            }
            else if (!clenRepository.VsichniClenove().Any(x => x.ClenId == id))
            {
                return View("StrankaNenalezena");
            }
            
                var pridaniDruzinyClenoviViewModel = new PridaniDruzinyClenoviViewModel()
                {
                    Clen = clenRepository.ClenPodleId(id),
                    Druziny = druzinaRepository.VsechnyDruziny().ToList()

                };
                pridaniDruzinyClenoviViewModel.Uzivatel = uzivatelRepository.UzivatelPodleId(pridaniDruzinyClenoviViewModel.Clen.SpravceId);
                return View(pridaniDruzinyClenoviViewModel);

        }
        [HttpPost]
        public IActionResult PridaniDruzinyClenovi(PridaniDruzinyClenoviViewModel pridaniDruzinyClenovi)
        {
            var doplneni = clenRepository.ClenPodleId(pridaniDruzinyClenovi.Clen.ClenId);
            switch (pridaniDruzinyClenovi.NazevNoveDruziny)
            {
                case "Modrásci":
                    doplneni.DruzinaId = 1;
                    break;
                case "Fialky":
                    doplneni.DruzinaId = 2;
                    break;
                case "Delfíni":
                    doplneni.DruzinaId = 3;
                    break;
                case "Chobotnice":
                    doplneni.DruzinaId = 4;
                    break;
            }
            clenRepository.EditaceClena(doplneni);
            //v zavislosti na provedene akci je vypsana informacni hlaska
            if (doplneni.DruzinaId == null)
            {
                unitOfWork.Commit();
                TempData["Zprava"] = "Družina odebrána.";
            }
            else
            {
                unitOfWork.Commit();
                TempData["Zprava"] = "Družina přidána.";
            }
            return RedirectToAction("Detail", new { @id = pridaniDruzinyClenovi.Clen.SpravceId });
        }
        public IActionResult EditaceClena(Guid id)
        {
            if (!MaPravo())
            {
                return View("PristupZakazan");
            }
            else if (!clenRepository.VsichniClenove().Any(x => x.ClenId == id))
            {
                return View("StrankaNenalezena");
            }

            var editaceClena = new EditaceClenaViewModel()
            {
                Clen = clenRepository.ClenPodleId(id),
                Druziny = druzinaRepository.VsechnyDruziny().ToList()
            };
            if (editaceClena.Clen.DruzinaId != null)
            {
                editaceClena.IdStareDruziny = editaceClena.Clen.DruzinaId;
            }

            //pouzit jiny formular pro zabraneni validaci jiz pri prvnim zobrazeni
                return View("EditaceClena2", editaceClena); 
           



        }

        [HttpPost]
        public IActionResult EditaceClena(EditaceClenaViewModel editaceClena)
        {
            if (!ModelState.IsValid)
            {
                return View("EditaceClena", editaceClena);
            }
            if (ModelState.IsValid)
            {
                ClenArchiv editovanyArchiv = clenArchivRepository.VsichniArchivniCleni().FirstOrDefault(x => x.ClenArchivId == editaceClena.Clen.ClenId);
                ClenArchiv pom = editovanyArchiv;
                pom.ClenArchivId = editaceClena.Clen.ClenId;
                pom.KrestniJmeno = editaceClena.Clen.KrestniJmeno;
                pom.Prijmeni = editaceClena.Clen.Prijmeni;



                editovanyArchiv = pom;
                clenArchivRepository.Aktualizace(editovanyArchiv);
               

                var doplneni = editaceClena.Clen;
                switch (editaceClena.NazevNoveDruziny)
                {
                    case "Modrásci":
                        doplneni.DruzinaId = 1;
                        break;
                    case "Fialky":
                        doplneni.DruzinaId = 2;
                        break;
                    case "Delfíni":
                        doplneni.DruzinaId = 3;
                        break;
                    case "Chobotnice":
                        doplneni.DruzinaId = 4;
                        break;
                    default:
                        break;
                }
                if (editaceClena.IdStareDruziny != doplneni.DruzinaId)
                {
                    foreach (var p in prihlaseniNaAkciClenRepository.VsechnaPrihlaseniNaAkci())
                    {
                        if (p.ClenId == doplneni.ClenId)
                        {
                            prihlaseniNaAkciClenRepository.OdebraniPrihlaseni(p.PrihlaseniNaAkciClenId);
                        }
                    }
                }
                clenRepository.EditaceClena(doplneni);
                unitOfWork.Commit();
                TempData["Zprava"] = "Člen úspěšně upraven.";
                return RedirectToAction("Detail", new { @id = editaceClena.Clen.SpravceId });

            }
            else
            {
                return View("EditaceClena");
            }

        }
        public IActionResult EditaceUzivatele(Guid id)
        {
            if (!(VratRoli() == "admin"))
            {
                return View("PristupZakazan");
            }
            else if (!uzivatelRepository.VsichniUzivatele().Any(x => x.Id == id))
            {
                return View("StrankaNenalezena");
            }
            int roleId = uzivatelRepository.UzivatelPodleId(id).RoleId;
            var editaceUzivatele = new EditaceUzivateleViewModel()
            {
                Uzivatel = uzivatelRepository.UzivatelPodleId(id),
                ByvalaRole = roleRepository.RolePodleId(roleId).Nazev
            };
            
                return View("EditaceUzivatele2", editaceUzivatele); 
            

        }
        [HttpPost]
        public IActionResult EditaceUzivatele(EditaceUzivateleViewModel editaceUzivatele)
        {
            if (!ModelState.IsValid)
            {
                return View("EditaceUzivatele", editaceUzivatele);
            }
            if (ModelState.IsValid)
            {
                int role = 0;
                switch (editaceUzivatele.ByvalaRole)
                {
                    case "člen":
                        role = 1;
                        break;
                    case "rodič":
                        role = 2;
                        break;
                    case "vedoucí":
                        role = 3;
                        break;
                    case "admin":
                        role = 4;
                        break;

                }
                var edit = editaceUzivatele.Uzivatel;
                //pokud byla rodici zmenena role, jsou smazani jeji clenove
                if (editaceUzivatele.ByvalaRole == "rodič" && role != editaceUzivatele.Uzivatel.RoleId)
                {
                    foreach (var c in clenRepository.ClenoveUzivatele(editaceUzivatele.Uzivatel.Id))
                    {
                        foreach (var p in prihlaseniNaAkciClenRepository.VsechnaPrihlaseniNaAkci())
                        {
                            if (p.ClenId == c.ClenId)
                            {
                                prihlaseniNaAkciClenRepository.OdebraniPrihlaseni(p.PrihlaseniNaAkciClenId);
                            }
                        }
                        clenRepository.OdebraniClena(c);
                    }
                }
                //pokud byvala role nebyla rodic, uzivatel je odebran ze vsech akci a je mu pripadne odebrana druzina
                if (editaceUzivatele.ByvalaRole != "rodič" && role != editaceUzivatele.Uzivatel.RoleId)
                {
                    editaceUzivatele.Uzivatel.DruzinaId = null;
                    foreach (var p in prihlaseniNaAkciClenRepository.VsechnaPrihlaseniNaAkci())
                    {
                        if (p.ClenId == editaceUzivatele.Uzivatel.Id)
                        {
                            prihlaseniNaAkciClenRepository.OdebraniPrihlaseni(p.ClenId);
                           
                        }
                    }
               
                }
                if (editaceUzivatele.Uzivatel.RoleId != 2 && editaceUzivatele.ByvalaRole != "člen")
                {
                    if (!clenArchivRepository.VsichniArchivniCleni().Any(x => x.ClenArchivId == editaceUzivatele.Uzivatel.Id))
                    {
                        ClenArchiv pom = new ClenArchiv()
                        {
                            ClenArchivId = editaceUzivatele.Uzivatel.Id,
                            KrestniJmeno = editaceUzivatele.Uzivatel.KrestniJmeno,
                            Prijmeni = editaceUzivatele.Uzivatel.Prijmeni
                        };
                        clenArchivRepository.PridaniArchivnihoClena(pom);
                    }

                }
                if ((editaceUzivatele.Uzivatel.RoleId == 1 && editaceUzivatele.ByvalaRole == "člen")
                    || (editaceUzivatele.Uzivatel.RoleId == 3 && editaceUzivatele.ByvalaRole == "vedoucí")
                    || (editaceUzivatele.Uzivatel.RoleId == 4 && editaceUzivatele.ByvalaRole == "admin"))
                {
                    ClenArchiv editovanyArchiv = clenArchivRepository.VsichniArchivniCleni().FirstOrDefault(x => x.ClenArchivId == editaceUzivatele.Uzivatel.Id);
                    ClenArchiv pom = editovanyArchiv;
                    pom.ClenArchivId = editaceUzivatele.Uzivatel.Id;
                    pom.KrestniJmeno = editaceUzivatele.Uzivatel.KrestniJmeno;
                    pom.Prijmeni = editaceUzivatele.Uzivatel.Prijmeni;



                    editovanyArchiv = pom;
                    clenArchivRepository.Aktualizace(editovanyArchiv);
                    
                }
                uzivatelRepository.EditaceUzivatele(edit);
                unitOfWork.Commit();
                TempData["Zprava"] = "Uživatel úspěšně upraven.";
                return RedirectToAction("Detail", new { @id = edit.Id });

            }
            else
            {
                return View("EditaceUzivatele");
            }

        }

    }
}