using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Infrastructure;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using ModrinkyWebApp.Models;
using ModrinkyWebApp.Models.Repositare;

namespace ModrinkyWebApp
{
    public class Startup
    {
        public IConfiguration Configuration { get; }

        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public void ConfigureServices(IServiceCollection services)
        {
            services.AddEntityFrameworkNpgsql()
                .AddDbContext<ModrinkyDbContext>(options =>
                    options.UseNpgsql(Configuration.GetConnectionString("ModrinkyConnection"))
                 );

            services.Configure<CookiePolicyOptions>(options =>
            {
                // This lambda determines whether user consent for non-essential cookies is needed for a given request.
                options.CheckConsentNeeded = context => true;
                options.MinimumSameSitePolicy = SameSiteMode.None;
            });

            services.AddMvc().SetCompatibilityVersion(CompatibilityVersion.Version_3_0);
            services.AddMvc(option => option.EnableEndpointRouting = false);

            services.AddTransient<IAktualitaRepository, AktualitaRepository>();
            services.AddTransient<IAkceRepository, AkceRepository>();
            services.AddTransient<IUzivatelRepository, UzivatelRepository>();
            services.AddTransient<IClenRepository, ClenRepository>();
            services.AddTransient<IPovolenaDruzinaRepository, PovolenaDruzinaRepository>();
            services.AddTransient<IDruzinaRepository, DruzinaRepository>();
            services.AddTransient<IPrihlaseniNaAkciUzivatelRepository, PrihlaseniNaAkciUzivatelRepository>();
            services.AddTransient<IPrihlaseniNaAkciClenRepository, PrihlaseniNaAkciClenRepository>();
            services.AddTransient<IPrihlaseniArchivRepository, PrihlaseniArchivRepository>();
            services.AddTransient<IClenArchivRepository, ClenArchivRepository>();
            services.AddTransient<IRoleRepository, RoleRepository>();
            services.AddTransient<IUnitOfWork, UnitOfWork>();
            services.AddSingleton<IActionContextAccessor, ActionContextAccessor>();

            services.AddMvc().AddSessionStateTempDataProvider();
            services.AddSession();
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseStaticFiles();
            app.UseStaticFiles();
            app.UseSession();
            app.UseCookiePolicy();
            app.UseAuthentication();

            app.UseMvc(routes =>
            {
                routes.MapRoute(
                    name: "default",
                    template: "{controller=Home}/{action=Index}/{id?}");
            });
        }
    }
}